package explorviz.hpc_monitoring.filter.reduction;

import java.util.*;
import com.lmax.disruptor.EventHandler;
import explorviz.hpc_monitoring.reader.RecordEvent;
import explorviz.hpc_monitoring.record.IRecord;
import explorviz.hpc_monitoring.record.Trace;
import explorviz.hpc_monitoring.record.events.AbstractOperationEvent;

public class TracePatternSummarizationFilter implements
        EventHandler<RecordEvent> {
    private final long                               maxCollectionDuration;
    private final Map<Trace, TraceAggregationBuffer> trace2buffer = new TreeMap<Trace, TraceAggregationBuffer>(
                                                                          new TraceComperator());

    public TracePatternSummarizationFilter(final long maxCollectionDuration) {
        this.maxCollectionDuration = maxCollectionDuration;
    }

    public void periodicTimeSignal(final Long timestamp) {
        processTimeoutQueue(timestamp);
    }

    private void processTimeoutQueue(final long timestamp) {
        final long bufferTimeout = timestamp - maxCollectionDuration;
        final List<Trace> toRemove = new ArrayList<Trace>();
        for (final TraceAggregationBuffer traceBuffer : trace2buffer.values()) {
            if (traceBuffer.getBufferCreatedTimestamp() <= bufferTimeout) {
                final Trace aggregatedTrace = traceBuffer.getAggregatedTrace();
                sendOutTrace(aggregatedTrace);
                toRemove.add(aggregatedTrace);
            }
        }
        for (final Trace traceEventRecords : toRemove) {
            trace2buffer.remove(traceEventRecords);
        }
    }

    private void sendOutTrace(final Trace aggregatedTrace) {
        // TODO
    }

    @Override
    public void onEvent(final RecordEvent event, final long sequence,
            final boolean endOfBatch) throws Exception {
        final IRecord value = event.getValue();
        if (value instanceof Trace) {
            insertIntoBuffer((Trace) value);
        }
    }

    private void insertIntoBuffer(final Trace trace) {
        TraceAggregationBuffer traceAggregationBuffer = trace2buffer.get(trace);
        if (traceAggregationBuffer == null) {
            traceAggregationBuffer = new TraceAggregationBuffer(
                    System.nanoTime());
            trace2buffer.put(trace, traceAggregationBuffer);
        }
        traceAggregationBuffer.insertTrace(trace);
    }

    public void terminate(final boolean error) {
        for (final TraceAggregationBuffer traceBuffer : trace2buffer.values()) {
            sendOutTrace(traceBuffer.getAggregatedTrace());
        }
        trace2buffer.clear();
    }

    private static final class TraceComperator implements Comparator<Trace> {

        @Override
        public int compare(final Trace t1, final Trace t2) {
            final AbstractOperationEvent[] recordsT1 = t1.getTraceEvents();
            final AbstractOperationEvent[] recordsT2 = t2.getTraceEvents();

            if ((recordsT1.length - recordsT2.length) != 0) {
                return recordsT1.length - recordsT2.length;
            }

            final int cmpHostnames = t1.getTraceMetadata().getHostname()
                    .compareTo(t2.getTraceMetadata().getHostname());
            if (cmpHostnames != 0) {
                return cmpHostnames;
            }

            // TODO deep check records
            return 0;
        }
    }
}
