package explorviz.hpc_monitoring.reader;

import java.util.concurrent.*;

public final class TimeReader {
    private final long                     period;

    private boolean                        terminated = false;

    private final ScheduledExecutorService executorService;
    private ScheduledFuture<?>             result;

    public TimeReader(final long periodInNanoSec) {
        period = periodInNanoSec;
        executorService = new ScheduledThreadPoolExecutor(1);
    }

    public void read() {
        result = executorService.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                sendTimestampEvent();
            }
        }, 0, period, TimeUnit.NANOSECONDS);
        try {
            result.get();
        }
        catch (final ExecutionException ex) {
            terminate();
        }
        catch (final InterruptedException ignore) {}
        catch (final CancellationException ignore) {}
        terminate();
    }

    protected void sendTimestampEvent() {
        // final long timestamp = System.nanoTime();
        // super.deliver(OUTPUT_PORT_NAME_TIMESTAMPS, timestamp);
    }

    public void terminate() {
        if (!terminated) {
            executorService.shutdown();
            try {
                terminated = executorService.awaitTermination(5,
                        TimeUnit.SECONDS);
            }
            catch (final InterruptedException ex) {}
            if (!terminated) {
                result.cancel(true);
            }
        }
    }
}
